#if !defined(IIIMP_PRINT_MISC_H)
#define IIIMP_PRINT_MISC_H

/* $Id: print-misc.h,v 1.1.1.1 2002/08/31 04:18:07 himi Exp $ */


#ifdef	__cplusplus
extern "C" {
#endif


extern const char *	opcode_string_get(int num);
extern const char *	feedback_string_get(int num);
extern const char *	object_descriptor_category_string_get(int num);
extern const char *	annotation_id_string_get(int num);
extern const char *	imattribute_id_string_get(int num);
extern const char *	icattribute_id_string_get(int num);
extern const char *	lookup_choice_master_string_get(int num);
extern const char *	lookup_choice_direction_string_get(int num);
extern const char *	lookup_choice_label_owner_string_get(int num);
extern const char *	lookup_choice_process_page_string_get(int num);
extern const char *	character_subsets_string_get(int num);


#ifdef	__cplusplus
}
#endif

#endif /* !IIIMP_PRINT_MISC_H */


/* Local Variables: */
/* c-file-style: "iiim-project" */
/* End: */
